<?php
namespace InternetGalerie\Igshop2\Domain\Model;

class Address extends \FriendsOfTYPO3\TtAddress\Domain\Model\Address {
    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $gender = '';

    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $firstName = '';

    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $lastName = '';

    /**
     * @var string
     */
    protected $company = '';

    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $address = '';

    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $zip = '';

    /**
     * @var string
     * @TYPO3\CMS\Extbase\Annotation\Validate("NotEmpty")
     */
    protected $city = '';

    /**
     * @var \InternetGalerie\Igshop2\Domain\Model\Country
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $country = null;

	/**
	 * moduleSysDmailHtml
	 *
	 * @var boolean
	 */
	protected $moduleSysDmailHtml = FALSE;

    /**
     * @var \InternetGalerie\Igshop2\Domain\Model\FeUser
     * @TYPO3\CMS\Extbase\Annotation\ORM\Lazy
     */
    protected $feUser = null;

    /**
     * @var boolean
     */
    protected $isBillingAddress = false;

    /**
     * @var boolean
     */
    protected $isPrivatePerson = false;

	/**
	 * Returns the moduleSysDmailHtml
	 *
	 * @return boolean $moduleSysDmailHtml
	 */
	public function getModuleSysDmailHtml() {
		return $this->moduleSysDmailHtml;
	}

	/**
	 * Sets the moduleSysDmailHtml
	 *
	 * @param boolean $moduleSysDmailHtml
	 * @return void
	 */
	public function setModuleSysDmailHtml($moduleSysDmailHtml) {
	  if( $this->moduleSysDmailHtml != $moduleSysDmailHtml)
	    $this->mustCalculateTotal=true;

	  $this->moduleSysDmailHtml = $moduleSysDmailHtml;
	}

	/**
	 * Returns the boolean state of moduleSysDmailHtml
	 *
	 * @return boolean
	 */
	public function isModuleSysDmailHtml() {
		return $this->moduleSysDmailHtml;
	}

    /**
     * Returns the feUser
     *
     * @return \InternetGalerie\Igshop2\Domain\Model\FeUser $feUser
     */
    public function getFeUser(){
        return $this->feUser;
    }

    /**
     * Sets the feUser
     *
     * @param \InternetGalerie\Igshop2\Domain\Model\FeUser $feUser
     */
    public function setFeUser($feUser){
        $this->feUser = $feUser;
    }

    /**
     * Returns the isBillingAddress
     *
     * @return boolean $isBillingAddress
     */
    public function getIsBillingAddress(){
        return $this->isBillingAddress;
    }

    /**
     * Sets the isBillingAddress
     *
     * @param boolean $isBillingAddress
     */
    public function setIsBillingAddress($isBillingAddress){
        $this->isBillingAddress = $isBillingAddress;
    }

    /**
     * Returns the isPrivatePerson
     *
     * @return boolean $isPrivatePerson
     */
    public function getIsPrivatePerson(){
        return $this->isPrivatePerson;
    }

    /**
     * Sets the isPrivatePerson
     *
     * @param boolean $isPrivatePerson
     */
    public function setIsPrivatePerson($isPrivatePerson){
        $this->isPrivatePerson = $isPrivatePerson;
    }

    /**
     * Updates the name field to [$firstName $lastName]
     */
    public function updateName() {
        $name = '';

        if($this->getFirstName() != '') {
            $name = $this->getFirstName();
        }

        if($this->getFirstName() != '' && $this->getLastName() != '') {
            $name .= ' ' . $this->getLastName();
        } else if($this->getLastName() != '') {
            $name = $this->getLastName();
        }

        $this->setName($name);
    }

    public function getName():string {
        if(parent::getName() == '') {
            $this->updateName();
        }

        return parent::getName();
    }

    /**
     * Returns a JSON representation of the address
     *
     * @return string $json
     */
    public function getJson() {
        return json_encode([
            'uid' => $this->getUid(),
            'email' => $this->getFeUser()->getEmail(),
            'phone' => $this->getFeUser()->getTelephone(),
            'gender' => $this->getGender() == 'm' ? 1 : ($this->getGender() == 'f' ? 2 : ''),
            'firstName' => $this->getFirstName(),
            'lastName' => $this->getLastName(),
            'company' => $this->getCompany(),
            'address' => $this->getAddress(),
            'zip' => $this->getZip(),
            'city' => $this->getCity(),
            'country' => is_object($this->getCountry()) ? $this->getCountry()->getUid() : $this->getCountry(),
            'isPrivatePerson' => $this->getIsPrivatePerson(),
        ], JSON_UNESCAPED_UNICODE);
    }
}
