<?php

namespace InternetGalerie\Igshop2\Utility;

use InternetGalerie\Igshop2\Domain\Repository\CountryRepository;
use InternetGalerie\Igshop2\Domain\Model\Currency;
use InternetGalerie\Igshop2\Domain\Repository\OrderRepository;
use InternetGalerie\Igshop2\Domain\Repository\OrderProductRepository;
use InternetGalerie\Igshop2\Domain\Repository\PortoRepository;
use InternetGalerie\Igshop2\Domain\Repository\ProductRepository;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface;
use TYPO3\CMS\Core\Context\Context;

class CartUtility implements \TYPO3\CMS\Core\SingletonInterface {
    /**
     * @var string Name of the extension this view helper belongs to
     */
    protected $extensionName = 'IgShop2';

    /**
     * @var string Name of the extension this view helper belongs to (ext_localconf.php configurePlugin zweites  Argument)
     */
    protected $pluginName = 'igshop2';
	/**
	 * orderRepository
	 *
	 * @var \InternetGalerie\Igshop2\Domain\Repository\OrderRepository
	 */
	protected $orderRepository = NULL;

    /**
     * productRepository
     *
     * @var \InternetGalerie\Igshop2\Domain\Repository\ProductRepository
     *
     **/
    protected $productRepository = NULL;

    /**
     * OrderProductRepository
     *
     * @var \InternetGalerie\Igshop2\Domain\Repository\OrderProductRepository
     */
    protected $orderProductRepository = NULL;

    /**
     * countryRepository
     *
     * @var \InternetGalerie\Igshop2\Domain\Repository\CountryRepository
     */
    protected $countryRepository = NULL;

	/**
	 * portoRepository
	 *
	 * @var \InternetGalerie\Igshop2\Domain\Repository\PortoRepository
	 */
	protected $portoRepository = NULL;


    /**
     * @var \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface
     */
    protected $configurationManager;

    /**
     * the current order
     *
     * @var \InternetGalerie\Igshop2\Domain\Model\Order
     */
    public $order=null;

    /**
     * country used just for default ? [DA] @todo remove
     *
     * @var \InternetGalerie\Igshop2\Domain\Model\Country
     */
    protected $country=null;

    protected $settings = null;

    /**
     * @param ConfigurationManagerInterface $configurationManager
     * @return void
     */
    public function injectConfigurationManager(ConfigurationManagerInterface $configurationManager)
    {
        $this->configurationManager = $configurationManager;
        $this->settings = $this->configurationManager->getConfiguration(ConfigurationManagerInterface::CONFIGURATION_TYPE_SETTINGS, $this->extensionName, $this->pluginName);
    }

    /**
     * @param OrderRepository $orderRepository;
     */
    public function injectOrderRepository(OrderRepository $orderRepository)
    {

        $this->orderRepository = $orderRepository;
    }
    /**
     * @param ProductRepository $productRepository;
     */
    public function injectProductRepository(ProductRepository $productRepository)
    {

        $this->productRepository = $productRepository;
    }
    /**
     * @param OrderProductRepository $orderProductRepository;
     */
    public function injectOrderProductRepository(OrderProductRepository $orderProductRepository)
    {

        $this->orderProductRepository = $orderProductRepository;
    }

    /**
     * @param CountryRepository $countryRepository;
     */
    public function injectCountryRepository(CountryRepository $countryRepository)
    {

        $this->countryRepository = $countryRepository;
    }

    /**
     * @param PortoRepository $portoRepository;
     */
    public function injectPortoRepository(PortoRepository $portoRepository)
    {

        $this->portoRepository = $portoRepository;
    }

    /**
     * Aktueller Warenkorb laden
     */
    public function initializeObject()
    {
        $this->country = $this->countryRepository->getCountryFromRemoteAddr($_SERVER['REMOTE_ADDR']);
        $currency = $this->country->getCurrency();
        $this->order= $this->orderRepository->getCurrentOrder($currency);
    }
    // OrderController.php ist doch in order drin??????
    public function getCountry()
    {
        return $this->country;
    }

    public function getCartByUid( $uid )
    {
        $this->order= $this->orderRepository->findByUid( $uid );
    }


    // von ProductController.php fuer Waehrung ???
    /*
    public function getCurrency(Currency $currency)
    {
        $this->getCart( $currency );
        return $this->order->getCurrency();
    }
    */

    public function addProduct(  $product, $options = [] )
    {
        $this->order->addNewProduct( $product,  $options, $this->settings['prodLimit']  ?? null);

    }
    public function addProductByUid( int $productUid,  $options = [])
    {
        $product = $this->productRepository->findByUid($productUid);
        $this->addProduct( $product, $options );
    }

    public function removeOrderProduct(  $orderProduct )
    {
        $this->order->removeProduct( $orderProduct );

    }
    public function removeOrderProductByUid( int $orderProductUid,  $options = [])
    {
        $orderProduct = $this->orderProductRepository->findByUid($orderProductUid);
        if( is_object($orderProduct)) {
            $orderProduct->setAmount(0);
            $this->orderProductRepository->update($orderProduct);
            $this->removeOrderProduct( $orderProduct);
        }
    }

    public function orderProductSetQuantity(  $orderProduct, $quantity )
    {
        return $this->order->changeProductQuantity( $orderProduct, $quantity );
    }

    public function orderProductSetQuantityByUid( int $orderProductUid,  $quantity)
    {
        $orderProduct = $this->orderProductRepository->findByUid($orderProductUid);
        if( is_object($orderProduct)) {
            return $this->orderProductSetQuantity( $orderProduct, $quantity );
        }
        return false;
    }
    // Bestellung ist mit Porto?
    public function hasPorto( )
    {
        $paymentMethod=$this->order->getPaymentMethod();
        // Ist Bestellung ohne Porto
        if(!$paymentMethod || $paymentMethod->getNoPorto() || $this->order->hasProductWithoutPorto() ) {
            return false;
        }
        return true;
    }

    public function setPorto( $portoUid )
    {
        if($this->hasPorto()) {
            $country=$this->order->getCountry();
            if( $portoUid>0) {
                $porto = $this->portoRepository->findByUid( $portoUid );
                if( is_object($porto) && $this->order->isPortoValidWithCountry( $country )) {
                    $this->order->setPorto($porto);
                    $this->update();
                    return;
                }  else {
                    // no valid porto -> Fehler ?!
                    //$this->order->removePorto();
                }
            }
            $this->setDefaultPorto( $country );
        } else {
            $this->order->removePorto();
        }
    }

    // Default Porto setzen falls leer -> wirklich noetig? [DA]
    public function setDefaultPorto( $country=null )
    {
        if( $country===null) {
            // @todo Mist - warum Porto mit Uid=1 [DA]  fuer OrderController showAction
            $porto = $this->portoRepository->findByUid( 1 );
        } else {
            // @todo neu $this->shop->getDefaultPortoForCountry($country)
            $porto = $this->portoRepository->getDefaultPortoForCountry( $country );
        }
        if( is_object($porto) ) {
                $this->order->setPorto($porto);
                $this->update();
        } else {
            // ist eh leer oder ungueltig?
            $this->order->removePorto();
        }
    }

    public function getAvailablePortos()
    {
        if($this->hasPorto()) {
            $country=$this->order->getCountry();
            // @todo neu $this->shop->getPortosForCountry($country)
            return $this->portoRepository->getAllForCountry($country);
        }
        return [];
    }
    public function setSubmitted( $submitted )
    {
        $this->order->setSubmitted( $submitted );
    }


    public function update( )
    {
        $this->orderRepository->persistOrder($this->order);

    }

}
